/*************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 *************************************************************************
 * MODULE: config.h
 * Site-dependent (but preferably not OS-dependent) definitions
 ************************************************************************/

/* #include "systems/sun4.h"	/* if the crap below doesn't work */

#if defined(__hpux)			/* HP/UX system */
#	include "systems/hpux9.h"
#endif
#if defined(__svr4__) || defined(SVR4)	/* svr4 system */
#  if defined(__sun__) || defined(sun)	/* assumes that: sun+svr4=solaris. */
#	include "systems/sol2.h"
#  endif
#else				/* BSD (at least, non-svr4) system */
#  if defined(__sun__) || defined(sun)
#	include "systems/sun4.h"
#  endif
#endif
#if defined(__linux__)
#	include "systems/linux.h"	/* Linux system */
#endif
#if defined(__sgi)
#	include "systems/irix5.h"	/* IRIX 5 system */
#endif

/*************************************************************
 * PATH DEFINES: (XPERIMENT version)
 * used to avoid screwing up normal operations.
 *
 * path names of the lockfiles created by lpd. Note that these
 * cannot reside on a SunOS/Solaris tmpfs partition, so /tmp
 * may not be a good location for them. This is because the lpd
 * process locks them, with fcntl(2), before proceeding.
 */
#ifdef sun
# define TMASTERLOCK 	"/share/tmp/lpd.lock."
# define TDEFLPDLOGF 	"/share/tmp/lpd.log."
# define TPRINTCAP 	"/share/tmp/printcap."
# define TPERMFILE  	"/share/tmp/printer_perms."
#else
# define TMASTERLOCK 	"/tmp/lpd.lock."
# define TDEFLPDLOGF 	"/tmp/lpd.log."
# define TPRINTCAP 	"/tmp/printcap."
# define TPERMFILE  	"/tmp/printer_perms."
#endif

/*************************************************************
 * program used by "lpd" to send mail once a file is printed.
 * It needs to be able to support one argument (the recipient),
 * and accept the message on stdin (including From: and
 * Subject: headers).
 *
 * If you don't define this, "/usr/lib/sendmail" will be used.
 */
/* #define MAIL "/usr/local/bin/smail"	/* if you use smail */

/*************************************************************
 * daemon user ID; usually either "daemon" or "lp".
 * If you don't define this, the system default will be used.
 */
#ifndef DAEMON
/* #define DAEMON	"lp"		/* SYSV */
/* #define DAEMON	"daemon"	/* BSD style */
#endif /* DAEMON */

/*************************************************************
 * PATH DEFINES: (REAL version)
 * path names of the lockfiles created by lpd.
 */
#define RMASTERLOCK	"/var/spool/lpd/lpd.lock."
#define RDEFLPDLOGF 	"/var/spool/lpd/lpd.log."

/* Comment out one or the other of these sets of defines.
 *
 * The first set of defines is set up so the "/var/spool/lpd"
 * directory can be shared using NFS, and all the printcap and
 * printer_perms files live in this directory, with their hostnames
 * appended.
 */
/* #define RPRINTCAP	"/var/spool/lpd/printcap."		/* */
/* #define RPERMFILE	"/var/spool/lpd/printer_perms."		/* */
/* #define APPEND_HOSTNAME_TO_CONFIGS				/* */

/*
 * The second set is the more traditional BSD setup; the printcap
 * and printer_perms files are kept in /etc for each host.
 */
#define RPRINTCAP	"/etc/printcap"				/* */
#define RPERMFILE	"/etc/printer_perms"			/* */
#undef APPEND_HOSTNAME_TO_CONFIGS				/* */

/*************************************************************
 * should printers allow non-text files to be printed?
 * careful: most forms of printer graphic control sequences
 * would be counted as "non-text" here.
 */
/* #define ALLOW_TEXT_ONLY				/* */

/*************************************************************
 * will file size limitations take effect?
 * if you define this, PLP will enforce a file size limitation
 * of 1000k on all printers, overridable from the printcap file.
 */
/* #define USE_MX					/* */

/*************************************************************
 * LD_LIBRARY_PATH and PATH env vars, so filters and prefilters
 * inherit a secure environment. LD_LIBRARY_PATH is used by
 * SunOS 4.1.x and Solaris (possibly also Linux) to provide
 * a search path for shared libraries.
 */
#define SECURE_PATH_ENV		\
    "PATH=/bin:/usr/bin:/usr/local/bin:/usr/etc:/usr/sbin:/etc:/sbin:/usr/ucb:/usr/bsd"

#define SECURE_LD_PATH_ENV	\
    "LD_LIBRARY_PATH=/lib:/usr/lib:/usr/ucblib:/usr/5lib:/usr/ccs/lib:/usr/openwin/lib"

/*************************************************************
 * CHECK_INCOMING_PORT and STRICT_INCOMING_PORT:
 *
 * if CHECK_INCOMING_PORT is defined, connections from ports
 * >= 1024 will be logged, as they may be forged lpr requests
 * and they violate the specification of the LPD protocol.
 *
 * if STRICT_INCOMING_PORT is defined, connections of this
 * sort will be refused outright.
 */

#define CHECK_INCOMING_PORT				/* */
#define STRICT_INCOMING_PORT				/* */

/*************************************************************
 * if SYMLINK_NEEDS_GROUP is defined, users must be members
 * of a certain group to use lpr's "-s" option. This group
 * is named in the "ln" option in the printcap file.
 */

/* #define SYMLINK_NEEDS_GROUP				/* */

/*************************************************************
 * if APPEND_TIME_TO_SYSLOG is defined, a timestamp is
 * appended to messages logged using syslog.
 * Most syslogd's do this anyway these days, so you 
 * probably don't need this.
 */

/* #define APPEND_TIME_TO_SYSLOG			/* */

/*************************************************************
 * DAEMON_GROUP is the name of the group that spool directories
 * have. This really doesn't matter, and it's only used in
 * "checkpc", but it does no harm to be able to change it
 * if you need to.
 */

#define DAEMON_GROUP    "daemon"

