#
# commands made by this script
#
COMMANDS= lpd lpr lpq lprm lpc pac checkpc lp lpstat

# source directory, holds all source
SRC=../src

SLIB=${SRC}/library
SCOM=${SRC}/common
SLPD=${SRC}/lpd
SLPR=${SRC}/lpr
SLPQ=${SRC}/lpq
SMISC=${SRC}/misc

# change this to use purify
#LINK= purify ${CC}
LINK= ${CC}

# lint directory, where the output of any lintings will go
LNT=../lint

LINTFLAGS= -huv ${FLAGS}
LINTFILTER = | sed -e '/warning: struct.union iovec never defined/d' -e '/:$$/d' \
	-e '/warning: possible pointer alignment problem/d'

COMPILE= ${CC} -c ${CFLAGS} 
CXREF=/usr/ucb/ctags -x
PRINT=cpr -r

CFLAGS= -I${SRC} -I. ${DEBUG} ${XPERIMENT} -DDAEMON=\"${DAEMON}\"

all: ${COMMANDS}
#
# Binary: Object dependencies
#
DEPlib =  \
	bsd-compat.o getopt.o link_support.o remote_ops.o startserver.o \
	checkperm.o getprinter.o lockfile.o setuid.o utils.o \
	errormsg.o global.o printcap.o setup_test.o
SRClib = \
	${SLIB}/bsd-compat.o ${SLIB}/getopt.o ${SLIB}/link_support.o \
	${SLIB}/remote_ops.o ${SLIB}/startserver.o ${SLIB}/checkperm.o \
	${SLIB}/getprinter.o ${SLIB}/lockfile.o ${SLIB}/setuid.o \
	${SLIB}/utils.o ${SLIB}/errormsg.o ${SLIB}/global.o \
	${SLIB}/printcap.o ${SLIB}/setup_test.o

libplp.a: $(DEPlib)
	ar ruv libplp.a $(DEPlib)
	$(RANLIB) libplp.a

DEPlpd =  \
	banner.o localprinter.o lpd.o print_support.o recvfiles.o remoteprinter.o \
	sendmail.o setstatus.o find_name.o print_stty.o startprinter.o \
	mexecv.o control_ops.o displayq.o setup_filter.o rmjob.o \
	servicereq.o libplp.a
SRClpd =  \
	${SLPD}/banner.c ${SLPD}/localprinter.c ${SLPD}/lpd.c \
	${SLPD}/print_support.c ${SLPD}/recvfiles.c ${SLPD}/remoteprinter.c \
	${SLPD}/sendmail.c ${SLPD}/setstatus.c ${SLPD}/find_name.c \
	${SLPD}/print_stty.c ${SLPD}/servicereq.c ${SLPD}/startprinter.c \
	${SCOM}/mexecv.c ${SCOM}/control_ops.c ${SCOM}/displayq.c \
	${SCOM}/setup_filter.c ${SCOM}/rmjob.c

lpd: $(DEPlpd)
	${LINK}  ${CFLAGS}  -o lpd $(DEPlpd) $(LIBS)

DEPlpr =  \
	lpr.o lpr_canprint.o lpr_filters.o lpr_global.o lpr_job.o \
	lpr_parms.o lpr_temp.o mexecv.o setup_filter.o libplp.a
SRClpr =  \
	${SLPR}/lpr.c ${SLPR}/lpr_canprint.c ${SLPR}/lpr_filters.c \
	${SLPR}/lpr_global.c ${SLPR}/lpr_job.c ${SLPR}/lpr_parms.c \
	${SLPR}/lpr_temp.c ${SCOM}/mexecv.c ${SCOM}/setup_filter.c

lpr: $(DEPlpr)
	${LINK}  ${CFLAGS}  -o lpr $(DEPlpr) $(LIBS)

DEPlpq = lpq.o displayq.o termclear.o libplp.a
SRClpq = ${SLPQ}/lpq.c ${SLPQ}/termclear.c ${SCOM}/displayq.c

lpq: $(DEPlpq)
	${LINK}  ${CFLAGS}  -o lpq $(DEPlpq) ${TERMLIB} $(LIBS)

DEPlpc = lpc.o control_ops.o libplp.a
SRClpc = ${SMISC}/lpc.c ${SCOM}/control_ops.c

lpc: $(DEPlpc)
	${LINK}  ${CFLAGS}  -o lpc $(DEPlpc) $(LIBS)

DEPlprm = lprm.o rmjob.o libplp.a
SRClprm = ${SMISC}/lprm.c ${SCOM}/rmjob.o 

lprm: $(DEPlprm)
	${LINK}  ${CFLAGS}  -o lprm $(DEPlprm) $(LIBS)

DEPcheckpc= checkpc.o libplp.a
SRCcheckpc= ${SMISC}/checkpc.c

checkpc: $(DEPcheckpc)
	${LINK}  ${CFLAGS}  -o checkpc $(DEPcheckpc) $(LIBS)

DEPpac= pac.o libplp.a
SRCpac= ${SRC}/pac.c 

pac: $(DEPpac)
	${LINK} ${CFLAGS}  -o pac $(DEPpac) $(LIBS)

lp: ../src/lp.sh
	cp ../src/lp.sh lp
	chmod 755 lp

lpstat: ../src/lpstat.sh
	cp ../src/lpstat.sh lpstat
	chmod 755 lpstat

# Object: Source  dependencies
#
control_ops.o:	${SCOM}/control_ops.c
	${COMPILE} ${SCOM}/control_ops.c
displayq.o:	${SCOM}/displayq.c
	${COMPILE} ${SCOM}/displayq.c
mexecv.o:	${SCOM}/mexecv.c
	${COMPILE} ${SCOM}/mexecv.c
rmjob.o:	${SCOM}/rmjob.c
	${COMPILE} ${SCOM}/rmjob.c
setup_filter.o:	${SCOM}/setup_filter.c
	${COMPILE} ${SCOM}/setup_filter.c
bsd-compat.o:	${SLIB}/bsd-compat.c
	${COMPILE} ${SLIB}/bsd-compat.c
checkperm.o:	${SLIB}/checkperm.c
	${COMPILE} ${SLIB}/checkperm.c
errormsg.o:	${SLIB}/errormsg.c
	${COMPILE} ${SLIB}/errormsg.c
getopt.o:	${SLIB}/getopt.c
	${COMPILE} ${SLIB}/getopt.c
getprinter.o:	${SLIB}/getprinter.c
	${COMPILE} ${SLIB}/getprinter.c
global.o:	${SLIB}/global.c
	${COMPILE} ${SLIB}/global.c
link_support.o:	${SLIB}/link_support.c
	${COMPILE} ${SLIB}/link_support.c
lockfile.o:	${SLIB}/lockfile.c
	${COMPILE} ${SLIB}/lockfile.c
printcap.o:	${SLIB}/printcap.c
	${COMPILE} ${SLIB}/printcap.c
remote_ops.o:	${SLIB}/remote_ops.c
	${COMPILE} ${SLIB}/remote_ops.c
setuid.o:	${SLIB}/setuid.c
	${COMPILE} ${SLIB}/setuid.c
setup_test.o:	${SLIB}/setup_test.c
	${COMPILE} ${SLIB}/setup_test.c
startserver.o:	${SLIB}/startserver.c
	${COMPILE} ${SLIB}/startserver.c
utils.o:	${SLIB}/utils.c
	${COMPILE} ${SLIB}/utils.c
banner.o:	${SLPD}/banner.c
	${COMPILE} ${SLPD}/banner.c
find_name.o:	${SLPD}/find_name.c
	${COMPILE} ${SLPD}/find_name.c
localprinter.o:	${SLPD}/localprinter.c
	${COMPILE} ${SLPD}/localprinter.c
lpd.o:	${SLPD}/lpd.c
	${COMPILE} ${SLPD}/lpd.c
print_stty.o:	${SLPD}/print_stty.c
	${COMPILE} ${SLPD}/print_stty.c
print_support.o:	${SLPD}/print_support.c
	${COMPILE} ${SLPD}/print_support.c
recvfiles.o:	${SLPD}/recvfiles.c
	${COMPILE} ${SLPD}/recvfiles.c
remoteprinter.o:	${SLPD}/remoteprinter.c
	${COMPILE} ${SLPD}/remoteprinter.c
sendmail.o:	${SLPD}/sendmail.c
	${COMPILE} ${SLPD}/sendmail.c
servicereq.o:	${SLPD}/servicereq.c
	${COMPILE} ${SLPD}/servicereq.c
setstatus.o:	${SLPD}/setstatus.c
	${COMPILE} ${SLPD}/setstatus.c
startprinter.o:	${SLPD}/startprinter.c
	${COMPILE} ${SLPD}/startprinter.c
lpq.o:	${SLPQ}/lpq.c
	${COMPILE} ${SLPQ}/lpq.c
termclear.o:	${SLPQ}/termclear.c
	${COMPILE} ${SLPQ}/termclear.c
lpr.o:	${SLPR}/lpr.c
	${COMPILE} ${SLPR}/lpr.c
lpr.h:	${SLPR}/lpr.h
	${COMPILE} ${SLPR}/lpr.h
lpr_canprint.o:	${SLPR}/lpr_canprint.c
	${COMPILE} ${SLPR}/lpr_canprint.c
lpr_filters.o:	${SLPR}/lpr_filters.c
	${COMPILE} ${SLPR}/lpr_filters.c
lpr_global.o:	${SLPR}/lpr_global.c
	${COMPILE} ${SLPR}/lpr_global.c
lpr_job.o:	${SLPR}/lpr_job.c
	${COMPILE} ${SLPR}/lpr_job.c
lpr_parms.o:	${SLPR}/lpr_parms.c
	${COMPILE} ${SLPR}/lpr_parms.c
lpr_temp.o:	${SLPR}/lpr_temp.c
	${COMPILE} ${SLPR}/lpr_temp.c
checkpc.o:	${SMISC}/checkpc.c
	${COMPILE} ${SMISC}/checkpc.c
lpc.o:	${SMISC}/lpc.c
	${COMPILE} ${SMISC}/lpc.c
lprm.o:	${SMISC}/lprm.c
	${COMPILE} ${SMISC}/lprm.c
pac.o:	${SMISC}/pac.c
	${COMPILE} ${SMISC}/pac.c

#
# Installation of commands
#
install: all
	-mv ${INSTALL_LIB}/lpd ${INSTALL_LIB}/lpd-
	${INSTALLCMD} -s -c -o root -m 755 lpd ${INSTALL_LIB}
	${INSTALLCMD} -s -c -o root -m 4711 lpq ${INSTALL_BIN}
	${INSTALLCMD} -s -c -o root -m 4711 lprm ${INSTALL_BIN}
	${INSTALLCMD} -s -c -o root -m 4711 lpr ${INSTALL_BIN}
	${INSTALLCMD} -c -o root -m 755 lp ${INSTALL_BIN}
	${INSTALLCMD} -c -o root -m 755 lpstat ${INSTALL_BIN}
	${INSTALLCMD} -s -c -o root -m 4711 lpc ${INSTALL_MAINT}
	${INSTALLCMD} -s -c -o root pac ${INSTALL_MAINT}
	${INSTALLCMD} -s -c -o root checkpc ${INSTALL_MAINT}
	( cd ../doc/man; ${MAKE} ${INSTALLCMD} "MAN=${INSTALL_MAN}" )

install-test: lpd lpq lpr
	${INSTALLCMD} -c -o root -m 755 lpd ${INSTALL_LIB}/lpd.new
	${INSTALLCMD} -c -o root -m 4711 lpq ${INSTALL_BIN}/lpq.new
	${INSTALLCMD} -c -o root -m 4711 lpr ${INSTALL_BIN}/lpr.new

remove:
	rm -f ${INSTALL_LIB}/lpd
	rm -f ${INSTALL_BIN}/lpq
	rm -f ${INSTALL_BIN}/lprm
	rm -f ${INSTALL_MAINT}/lpc
	rm -f ${INSTALL_BIN}/lpr
	rm -f ${MAINTDIR}/pac
	rm -f ${MAINTDIR}/checkpc
#
# Lint dependencies
#

lint:	$(LNT)/lpd $(LNT)/lpr $(LNT)/lpq $(LNT)/lprm \
		$(LNT)/lpc $(LNT)/checkpc $(LNT)/pac

$(LNT)/lpd: $(SRClpd)
	lint  ${LINTFLAGS} $(SRClpd)  ${LINTFILTER} > $(LNT)/lpd

$(LNT)/lpr: $(SRClpr)
	lint  ${LINTFLAGS} $(SRClpr)  ${LINTFILTER} > $(LNT)/lpr

$(LNT)/lpq: $(SRClpq)
	lint  ${LINTFLAGS} $(SRClpq)  ${LINTFILTER} > $(LNT)/lpq

$(LNT)/lprm: $(SRClprm)
	lint  ${LINTFLAGS} $(SRClprm)  ${LINTFILTER} > $(LNT)/lprm

$(LNT)/lpc: $(SRClpc)
	lint  ${LINTFLAGS} $(SRClpc)  ${LINTFILTER} > $(LNT)/lpc

$(LNT)/checkpc: $(SRCcheckpc)
	lint  ${LINTFLAGS} $(SRCcheckpc)  ${LINTFILTER} > $(LNT)/checkpc

$(LNT)/pac: $(SRCpac)
	lint  ${LINTFLAGS} $(SRCpac)  ${LINTFILTER} > $(LNT)/pac

#
# Miscellaneous
#
clean:
	-rm -f ${COMMANDS} core *.o *.a

print:
	@${PRINT} Makefile *.h ${SRCS} | lpr -J lpd

xref:
	@${CXREF} *.c | pr -h XREF

Tags:
	ctags -t *.c *.h >.ctags_warnings 2>&1
perms:
	chown root lp? lp??
	chmod  u=sxr,g=xr,o=xr lp? lp??
unperms:
	for i in ${COMMANDS} ; \
	do \
	case $$i in \
		lp? | lp??)	\
			echo $$i ;	\
			cat $$i >x; rm $$i ; mv x $$i; \
			chmod a+x $$i	;	\
			;;	\
	esac				\
	done

test:
	-rm checkpc.o lpc.o lpd.o lpq.o lpr.o lprm.o pac.o
	make XPERIMENT=-DXPERIMENT=1 all

#***************************************************************************
#* Revision History:  Recreated
#* Sun Jan  3 07:27:44 CST 1988: V1
#***************************************************************************
#$Log:	Makefile,v $
# Revision 3.2  88/06/24  17:58:54  papowell
# MODS for VAX 4.3BSD UNIX
# 
# Revision 3.1  88/06/18  09:33:48  papowell
# Version 3.0- Distributed Sat Jun 18 1988
# 
# Revision 3.0  88/06/18  09:29:46  papowell
# Version 3.0- Distributed Sat Jun 18 1988
# 
# Revision 2.3  88/05/16  12:09:58  papowell
# Added a "test" entry to make XPERIMENT version
# 
# Revision 2.2  88/05/11  09:53:19  papowell
# Modified installation targets to be compatible with Berkeley LPR
# 
# Revision 2.1  88/05/09  10:07:22  papowell
# PLP: Released Version
# 
# Revision 1.10  88/04/29  07:44:51  papowell
# Added USE_LOCKF for alternative form of locking
# 
# Revision 1.9  88/04/28  17:30:23  papowell
# Fixed installation
# 
