/*******************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 *******************************************************************
 * Some stuff for Solaris and other SVR4 impls; emulate BSD sm'tics.
 ******************************************************************/

#include "lp.h"

/**************************************************************/
#ifdef USE_SIGPROCMASK

static sigset_t oblock;

void plp_block_signals() {
        sigset_t block, oblock;

        (void)sigemptyset(&block);
        (void)sigaddset(&block, SIGCHLD);
        (void)sigaddset(&block, SIGHUP);
        (void)sigaddset(&block, SIGINT);
        (void)sigaddset(&block, SIGQUIT);
        (void)sigaddset(&block, SIGTERM);
        if (sigprocmask(SIG_BLOCK, &block, &oblock) < 0)
                perror("sigprocmask");
}

void plp_unblock_signals() {
        (void)sigprocmask(SIG_SETMASK, &oblock, (sigset_t *)NULL);
# if defined(SA_RESTART) && defined(NEED_RESTARTABLE_SYSCALLS)
	setup_restartable_syscalls();
# endif  /* SA_RESTART && NEED_RESTARTABLE_SYSCALLS */
}

/**************************************************************/
#else	/* BSD style */

static int omask;

void plp_block_signals() {
        omask = sigblock(sigmask(SIGCHLD)|sigmask(SIGHUP)
                |sigmask(SIGINT)|sigmask(SIGQUIT)|sigmask(SIGTERM));
}

void plp_unblock_signals() {
        (void)sigsetmask(omask);
}
#endif
/**************************************************************/

#if defined(SA_RESTART) && defined(NEED_RESTARTABLE_SYSCALLS)

/* make system calls restartable, BSD style, instead of returning EINTR */

void setup_restartable_syscalls() {
        struct sigaction act, oact;
        act.sa_flags |= SA_RESTART;
        if (sigaction(SIGCHLD, &act, &oact) < 0) perror("sigaction");
        if (sigaction(SIGHUP, &act, &oact) < 0) perror("sigaction");
        if (sigaction(SIGINT, &act, &oact) < 0) perror("sigaction");
        if (sigaction(SIGQUIT, &act, &oact) < 0) perror("sigaction");
        if (sigaction(SIGTERM, &act, &oact) < 0) perror("sigaction");
}
#endif

/*
 * this one is tough -- setuid handling varies pretty widely
 * from UNIX to UNIX. oh well.
 */
#ifdef USE_SETUID
int setreuid (ruid, euid)
  int ruid, euid;
{
  if (ruid != -1) {
    if (setuid(ruid) != 0)
      logerr_die( XLOG_INFO, "error in setuid: %s", Errormsg(errno));
  }

#ifdef NO_SETEUID
  if (euid != -1) {
    if (seteuid(euid) != 0)
      logerr_die( XLOG_INFO, "error in seteuid: %s", Errormsg(errno));
  }
#endif

  return 0;
}
#endif

#ifdef USE_GETCWD
char *getwd(path)
  char *path;
{
  /* MAXPATHLEN -- sys/param.h is included by lp.h */
  if (getcwd (path, MAXPATHLEN)) {
    return path;
  } else {
    char *x;

    x = Errormsg(errno); strcpy (path, x);
    return (char *) NULL;
  }
}
#endif
