/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 ***************************************************************************
 * MODULE: Remote_ops.c
 * intiate remote operations from the local machine
 ***************************************************************************/

#include "lp.h"

/***************************************************************************
 * Remote_status()
 * get the status from the remote Host RM for Printer RP
 ***************************************************************************/
Remote_status()
{
	char buf[BUFSIZ];

	if( RP == 0 || *RP == 0 ){
		fatal(XLOG_INFO, "no RP specified for RM (%s)", RM );
	}
	(void)sprintf( buf, "%c%s\n",  Short_format? REQ_DSHORT : REQ_DLONG,RP);
	if(Debug>3)log(XLOG_DEBUG,"Remote_status: %s, '%d'%s",RM,buf[0],buf+1);
	if( JSUCC == Link_line( 3, buf )){
		if(Debug>4)log(XLOG_DEBUG,"request sent");
		Link_get();
	} else {
		(void)fprintf( stdout, "Remote connect to '%s' (%s) failed: %s\n",
			RP, RM, Errormsg(errno));
	}
	Link_close();
}

/***************************************************************************
 * Remote_remove()
 * send a remote remove request to the remote Host
 ***************************************************************************/
Remote_remove()
{
	char buf[BUFSIZ];		/* holds command */
	char *bp, *ep;			/* ACME Pointers, Inc. */
	int i;					/* ACME Integers, Inc. */
	static char *xsp = " ";	/* needs a space */

	/*
	 * set up the remote command format:
	 * <REQ_REMOVE>Printer Person [Parms]
	 */
	bp = buf; ep = buf+sizeof(buf);
	*bp++ = REQ_REMOVE;
	if( RP == 0 || *RP == 0 ){
		fatal( XLOG_INFO, "no RP specified for RM %s", RM );
	}
	bp = estrcp( bp, RP, ep );
	bp = estrcp( bp, xsp, ep );
	bp = estrcp( bp, Person, ep );
	for( i = 0; i < Parmcount; ++i ){
		bp = estrcp( bp, xsp, ep );
		bp = estrcp( bp, Parms[i].str, ep );
	}
	bp = estrcp( bp, "\n", ep );
	/*
	 * check command line for length
	 */
	if( bp == 0 ){
		fatal( XLOG_INFO, "Remote_remove: command line too long '%s'");
	}
	/*
	 * send command
	 */
	if(Debug>3)log(XLOG_DEBUG,"Remote_remove: %s, '%d'%s",RM,buf[0],buf+1);
	if( JSUCC == Link_line( 3, buf )){
		if(Debug>3)log(XLOG_DEBUG,"request sent");
		Link_get();
	}
	Link_close();
}

/***************************************************************************
 * Remote_control(cmd)
 * send the remote command to the printer
 * This has the form: \006person command RP
 ***************************************************************************/
Remote_control(cmd)
	char *cmd;
{
	char buf[BUFSIZ];

	if( RP == 0 || *RP == 0 ){
		fatal(XLOG_INFO, "no RP specified for RM (%s)", RM );
	}
	(void)sprintf( buf, "%c%s %s %s\n",  REQ_CONTROL,Person,cmd,RP);
	if(Debug>3)log(XLOG_DEBUG,"Remote_control:%s '%d%'%s'",RM,buf[0],buf+1);
	if( JSUCC == Link_line( 3, buf )){
		if(Debug>4)log(XLOG_DEBUG,"request sent");
		Link_get();
	} else {
		(void)fprintf( stdout, "Remote connect to '%s' (%s) failed: %s\n",
			RP, RM, Errormsg(errno));
	}
	Link_close();
}

/***************************************************************************
 * Remote_start()
 * start the remote printer
 ***************************************************************************/
Remote_start()
{
	char buf[BUFSIZ];
	int succ;

	if( RP == 0 || *RP == 0 ){
		fatal(XLOG_INFO, "no RP specified for RM (%s)", RM );
	}
	(void)sprintf( buf, "%c%s\n", REQ_START, RP);
	if(Debug>3)log(XLOG_DEBUG,"Remote_start:%s '%d%'%s'",RM,buf[0],buf+1);
	succ = Link_line( 3, buf );
	if( JSUCC == succ){
		if(Debug>4)log(XLOG_DEBUG,"request sent");
		(void)Link_confirm();
	}
	Link_close();
	return( JSUCC == succ );
}
