/*************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 *************************************************************************
 * MODULE: lp.h
 * Includes and definitions for all programs
 *************************************************************************/

/*************************************************************************
 * Make sure that you set the DEBUG flag if XPERIMENT is set
 *
 *************************************************************************/

#include "config.h"

#define DEBUG

/******************************************************************
 * System Include Files
 ******************************************************************/
#include <unistd.h>
#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <netinet/in.h>
#include <netdb.h>
#include <pwd.h>
#include <signal.h>
#include <sys/wait.h>
#include <ctype.h>
#include <errno.h>
#include <grp.h>

#ifdef USE_DIRENT
#include <dirent.h>
#else
#include <sys/dir.h>
#endif

#ifdef NO_BFUNCTIONS
#define bcopy(x,y,len)	(memmove(y,x,len))
#define bzero(x,len)	(memset(x,0,len))
#define bcmp(x,y,len)	(memcmp(x,y,len))
#endif

#ifndef USE_INDEX
#define index(x,y)	((char *) strchr(x,y))
#define rindex(x,y)	((char *) strrchr(x,y))
#endif

#ifdef USE_FCNTL_LOCK
#include <sys/fcntl.h>
#endif

/******************************************************************
 * Declare Standard Routines.
 * Note: most of these are not declared in include files.
 * This leads to some interesting problems with LINT. Declare these
 * here.  Some specialized routines are declared in the individual
 * routines that use them.
 ******************************************************************/
#ifdef NO_MALLOC_H
extern char	*malloc(), *realloc();
extern int      free();
#else
#include <malloc.h>
#endif

#ifdef DECLARE_SPRINTF
extern char *sprintf();	/* sigh... sprintf strikes again */
#endif

extern char	*getenv();
extern char	*gets();
extern char *strchr(), *strrchr();
extern int errno, sys_nerr;
extern char *sys_errlist[];

extern long atol();
extern off_t lseek();
extern size_t fread(), fwrite();
extern pid_t getpid(), getppid(), fork();
extern uid_t getuid(), geteuid();
extern time_t time();	/* watch this one */

/*************************************************************************
 * PRINTCAP entries and associated variables.
 * Each printcap entry has a corresponding variable; for example
 * ":af=accnt:"
 *   -> AF = "accnt";
 *************************************************************************/
extern int    AB;		/* Always have a banner flag, ignore lpr -h option */
extern char  *AF;		/* accounting file */
extern int    AL;		/* remote annex serial port number */
extern char  *BP;		/* banner printer filter */
extern int    BK;		/* Berkeley compatible remote and local */
extern int    BR;		/* baud rate if lp is a tty */
extern char  *CM;		/* status line comment */
extern int    CO;		/* cost of printing in dollars */
extern int    DB;		/* Set debug level for queue handler */
extern char  *EP;		/* end printer filter */
extern int    FD;		/* No forwarding, accept only from original site */
extern int    FO;		/* print a form feed when device is opened */
extern int    FC;		/* flags to clear if lp is a tty */
extern char  *FF;		/* form feed string */
extern int    FJ;		/* send control file  first to remote site */
extern int    FQ;		/* form feed on quitting */
extern int    FS;		/* flags to set if lp is a tty */
extern char  *FX;		/* allowable formats */
extern char  *LD;		/* leader string on opening */
extern char  *LF;		/* log file for error messages */
extern int    LH;		/* use long host name */
extern char  *LO;		/* lock file name */
extern char  *LN;		/* group allowed to use links */
extern char  *LP;		/* line printer device name */
extern int    MC;		/* maximum number of copies allowed */
extern int    ML;		/* min legal chars for printable */
extern char  *MS;		/* new stty options - as per sun */
extern int    MX;		/* maximum number of blocks to copy */
extern int    NW;		/* Networked file system, do not make copies */
extern int    PL;		/* page length */
extern int    PO;		/* remote port to connect to */
extern char  *PR;		/* pr program name name */
extern char  *PS;		/* printer status file name */
extern int    PW;		/* page width */
extern int    PX;		/* page width in pixels */
extern int    PY;		/* page length in pixels */
extern char  *QH;		/* queue handler */
extern char  *RA;		/* remote annex name */
extern char  *RG;		/* restrict use to group */
extern char  *RM;		/* remote machine name */
extern char  *RP;		/* remote printer name */
extern int    RT;		/* max retries */
extern int    RW;		/* open LP for reading and writing */
extern int    SB;		/* short banner instead of normal header */
extern int    SC;		/* suppress multiple copies */
extern char  *SD;		/* spool directory */
extern int    SF;		/* suppress FF on each print job */
extern int    SH;		/* suppress header page */
extern char  *SS;		/* name of queue that the server serves  */
extern char  *ST;		/* status file name */
extern char  *SV;		/* names of servers */
extern char  *TR;		/* trailer string to be output when Q empties */
extern char  *TY;		/* terminal characteristics, stty options */
extern int    XC;		/* flags to clear for local mode */
extern int    XS;		/* flags to set for local mode */
extern char  *XT;		/* format checked for text only */
extern char  *XU;		/* restrict use to users whose names are in file */
/*
 * Filter_name[] is an array of filter names, indexed by the first letter
 * of the printcap tag. For example:
 * ":if=/usr/lib/iffilter -n -a:"
 *     -> Filter_name['i'-'a'] = "/usr/lib/iffilter -n -a";
 */
extern char *Filter_name[26];
#define	IF	(Filter_name['i'-'a'])
#define	OF	(Filter_name['o'-'a'])
/*
 * Prefilter_name[] is similar to Filter_name[], but for prefilter entries.
 */
extern char *Prefilter_name[26];

/* this is much smaller than MAXHOSTNAMELEN, but it's needed -- plp_queue
 * can only take params <=80 chars in length, and that includes the
 * submitting hostname.
 */
#define HOSTLEN		64

/**********************************************************************
 * Globally accessed variables and routines
 **********************************************************************/
extern char	*Name;				/* program name */
extern char	*Printer;			/* printer name */
extern char	*Person;			/* user name */
extern char	Host[HOSTLEN];			/* Host machine name */
extern char	*From;				/* client's machine name */
extern int	Debug;				/* Debugging level */
extern int	Echo_on_stdout;		/* stderr output on stdout as well */
extern char *First_name;		/* first name in printcap entry */
extern FILE *Lfd;				/* lock file */
extern int Optind, Opterr;		/*  option index, error message suppression */
extern char *Optarg;			/*  option value */
extern char *Opt_flag;			/*  option staring flags */
extern char Lpdlogf[];		/* = DEFLPDLOGF; */
extern char Masterlock[];		/*  = MASTERLOCK; */
extern int	Lpr_port_num;		/* port number to connect to */
extern int Maxportno;			/*  = MAXPORTNO; */
extern int Minportno;			/*  = MINPORTNO; */
extern char Permfile[];			/*  = PERMFILE; */
extern char Printcap[];			/*  = PRINTCAP; */
extern char Last_errormsg[];	/* last errormessage */
extern int Request;				/* current lpd request code */
extern int Print_fd;			/* output printer FILE */
extern int Short_format;		/* short status format */
extern int  Is_root;			/* is this the root user? */
extern int  Is_local;			/* doing this locally or remotely? */
extern struct stat LO_statb;	/* used to stat lockfile */
extern int Daemon_uid;			/* daemon UID */
extern int Daemon_gid;			/* daemon GID */
extern char *estrcp();			/* handy form of strcp() */
extern char *Time_str();		/* ctime with \n removed */
extern char *Setup_filter();	/* set up argv for a filter */
extern SIG_RETURN_TYPE Reapchild();	/* wait3() for a child */
extern int Getopt();			/* get command line option */
extern char *Errormsg();		/* printable error message */
extern int Errorcode;			/* exit() value on fatal termination */
extern char *Decode_status();	/* decode a status returned by wait() */
extern int Checkperm();		/* check permissions */

/*******************************************************************
 *  struct plp_queue
 *    used to record entries in the spool directory for printing
 *    and status reporting.
 *  NOTE: control and data file names in spool directory have the form
 *  c f <priority> <job number> <Hostname>  <terminating 0>
 *  d f <seq>      <job number> <Hostname>  <terminating 0>
 *  1 1 1               3        HOSTLEN        1
 * eg: dfA001attila.cs.umn.edu         (attila the SUN, of course)
 * eg: cfZ001attila.cs.umn.edu         (attila the SUN, of course)
 *******************************************************************/
#define CFNAMELEN (sizeof(Host)+8)	/* control or data file name length */
/*
 * fields in the name
 */
#define STARTPR	2	/* df[] */
#define STARTID	3	/* dfX[] */
#define IDLEN	3	/* dfX[NNN] */
#define	STARTFR	6	/* dfXNNN[] */
#define MAXPARMLEN	80
struct plp_queue {
	char	q_name[CFNAMELEN+1];		/* name of the control file */
	time_t	q_time;				/* modification time */
	time_t	q_sp;				/* spooled at time */
	time_t	q_unsp;				/* unspool at time */
	int	q_priority;			/* priority */
	long	q_size;				/* size of the data files */
	char	q_user[32];			/* user name */
	int	q_num;				/* job number */
	char	q_data[MAXPARMLEN+1];		/* data files name */
	int	q_daemon;			/* daemon for this entry */
	char	*q_server;			/* server for this entry */
};

#define	q_from	q_name[STARTFR]
extern int Getq();			/* find queue entries */
extern struct plp_queue *Jobentry;	/* current job entry in queue */
extern struct plp_queue *Queue;		/* Getq generates an array of jobs */
extern int Jobcount;			/* and returns the number of jobs  */

extern int Rec_cnt;					/* number of files in a job */

/********************************************************************
 * File Permissions
 ********************************************************************/

#define LFMODE      0644    /* log file permissions */
#define FILMOD      0600	/* spooling directory file permission */

/*
 * Queue Control is done by using the permissions of the lock file
 * for the queue.  The following permissions are used:
 * 	Disable Printing: owner execute or 0100
 * 	Disable Queing:	group execute or 0010
 * 	Queue handler active (antique) 0001
 */
#define	DISABLE_PRINT	(0100)
#define	ENABLE_PRINT	(0777 & ~ DISABLE_PRINT)
#define	DISABLE_QUEUE	(0010)
#define	ENABLE_QUEUE	(0777 & ~ DISABLE_QUEUE)
#define	FORCE_REQUE		(0001)
#define	CLEAR_REQUE		(0777 & ~ FORCE_REQUE)

/*
 *  syslog(8) message priorities.  These are defined by the values of
 *  global variables.  The variables are initialized either with values
 *  in the <syslog.h> include file,  or by a set of default values according
 *  to the NOSYSLOG option in the Makefile.
 */
extern int XLOG_ERR;			/*4 synonym of LOG_ERROR */
extern int XLOG_CRIT;			/*5	critical information */
extern int XLOG_WARNING;		/*6	warning */
extern int XLOG_NOTICE;			/*7	important information */
extern int XLOG_INFO;			/*8	informational message */
extern int XLOG_DEBUG;			/*9	Debug level info */

/*******************************************************************
 * Printcap Data Structures
 * The printcap file information is extracted using a single pass of
 * the printcap database.  As each entry is found,  it is looked up
 * in a table which has the entry tag,  the type of entry,
 * a default value,  and the variable used to hold the value.
 *
 * The table is sorted alphabetically by the tag values and the update
 * routines use a modified binary search.  The time taken to read the
 * printcap entry is proportional to
 * K  log( M ), where K is the length of the printcap file and M is
 * the number of printcap variables.  Note that this appears to be
 * a much simpler and easier way to implement printcap information
 * extraction.  The "initialization" of the entire set of variables
 * is extremely fast,  apparently taking about the same time  as reading
 * two variables using the original "termcap" code.
 *******************************************************************/
typedef struct pc_entry{
	char pc_name[3];	/* two character name, and the last char */
	int	kind;	/* the kind of entry */
#		define PC_NUM		0	/* integer */
#		define PC_FLAG		1	/* integer */
#		define PC_STRING	2	/* string */
	int idefault;	/* this is the default integer or flag value */
	char *sdefault;	/* this is the default integer or flag value */
	/*
	 * the variable is assumed to be a pointer to char;
	 * this is cast to pointer to int if neccessary. 
	 */
	char **var;
} PC_ENTRY;

/*
 * Routines to look up printer names and printcap entries
 * char ** All_printers():  reads the printcap database and extracts the
 *   names of all the printer entries.
 * char *First_printer(): gets name of first printer
 * Get_pc_entry(): given name of a printer and an array  of PC_ENTRIES,
 *  it finds the printcap entry and extracts all the information
 *  for the variables listed in the array.
 * Set_pc_entry(): same functionality, does not initialize variables.
 */
#define PRNAMELEN	32	/* Maximum of 32 characters for printer name */
#define MAXPCNAMES	100	/* Maximum of 100 printers */
extern int Get_pc_entry( /* char *name; PC_ENTRY *pc_vars; int pc_len */);
extern int Set_pc_entry( /* char *name; PC_ENTRY *pc_vars; int pc_len */);
extern char **All_printers();	/* returns array printer names */
extern char *First_printer();	/* returns array printer names */
/*
 * All_pc_vars[]: all printcap variables, and is used by lpr and lpd;
 * Status_pc_vars[]: a short set needed by lpq, lprm, and lpc.
 * Server_pc_vars[]: a few needed by servers only
 */
extern PC_ENTRY All_pc_vars[ /* All_pc_len */ ];
extern int	All_pc_len;
extern PC_ENTRY Status_pc_vars[ /* Status_pc_len */ ];
extern int	Status_pc_len;
extern PC_ENTRY Server_pc_vars[ /* Server_pc_len */ ];
extern int	Server_pc_len;

/***********************************************************************
 * File Locking Support:
 * see lockfile.c for details.
 ***********************************************************************/
extern FILE *Readlockfile();	/* reads file, returns FILE * */
extern FILE *Getlockfile();		/* locks file, returns FILE * */
extern int   Checklockfile();	/* checks for lock and daemon */
extern FILE *Lockcf();			/* lock a control file */
extern int   Exlockcf();		/* create and lock a control file */
extern void Setlockfile();	/* set a lock on printer */
extern void Closelockfile();	/* close lock file */

extern FILE *fopen_daemon();	/* open a file as daemon */
extern int open_daemon();		/* open a file as daemon */
extern int chmod_daemon();	/* change file modes */
extern int unlink_daemon();	/* delete a file */
extern int mkdir_daemon();	/* make a dir */

/***********************************************************************
 * Parameter List Support
 * struct parm parmlist[]
 *  is used when parsing a command received from a remote Host
 * It is also used by lpr to determine the command file, and to keep
 * track of the file/datafile name correspondence.
 ***********************************************************************/
#define MAXPARMS 50
struct parm{
	char *str;					/* string parameter */
	int   num;					/* number parameter */
	char filename[CFNAMELEN+1];	/* file name for parameter */
	long size;					/* size of file */
};
extern struct parm Parms[MAXPARMS];		/* array of parmaters */
extern int    Parmcount;				/* number of parameters */

/***********************************************************************
 * Request types
 * A request sent to the LPD daemon has the format:
 * \Xprinter [options],  where \X is a single character or byte value.
 * The following are the values and commands
 ***********************************************************************/
#define REQ_START	1	/* start printer */
#define REQ_RECV	2	/* transfer a job to the Host from a remote site */
#define REQ_DSHORT	3	/* print short form of queue status */
#define REQ_DLONG	4	/* print long form of queue status */
#define REQ_REMOVE	5	/* remove jobs */
#define REQ_CONTROL	6	/* do control operation */
/**************************************************************************
 *      Control file format
 *      First character is kind of entry, remainder of line is
 *          the argument.
 *
 *		1 -- "R font file" for troff -ignore
 *		2 -- "I font file" for troff -ignore
 *		3 -- "B font file" for troff -ignore
 *		4 -- "S font file" for troff -ignore
 *		C -- "class name" on banner page
 *		D -- "Date" job submitted
 *		H -- "Host name" of machine where lpr was done
 *		I -- "indent" amount to indent output
 *		J -- "job name" on banner page
 *		L -- "literal" user's name to print on banner
 *		M -- "mail" to user when done printing
 *		N -- "name" of file (used by lpq)
 *		P -- "Person" user's login name
 *		R -- account id  for charging 
 *		U -- "unlink" name of file to remove after we print it
 *		W -- "width" page width for PR
 *		X -- "header" header title for PR
 *		Z -- xtra options to filters 
 *
 *		Lower case letters are formats
 *		f -- "file name" name of text file to print
 *		l -- "file name" text file with control chars
 *		p -- "file name" text file to print with pr(1)
 *		t -- "file name" troff(1) file to print
 *		n -- "file name" ditroff(1) file to print
 *		d -- "file name" dvi file to print
 *		g -- "file name" plot(1G) file to print
 *		v -- "file name" plain raster file to print
 *		c -- "file name" cifplot file to print
 *
 * CFparm is used to hold the upper case parameters
 */
extern char CFparm[26][MAXPARMLEN+1];
#define CLASSNAME CFparm['C'-'A']
#define WHENSP    CFparm['D'-'A']
#define FROMHOST  CFparm['H'-'A']
#define INDENT    CFparm['I'-'A']
#define JOBNAME   CFparm['J'-'A']
#define BNRNAME   CFparm['L'-'A']
#define FILENAME  CFparm['N'-'A']
#define MAILNAME  CFparm['M'-'A']
#define LOGNAME   CFparm['P'-'A']
#define ACCNTNAME CFparm['R'-'A']
#define NOHEADER  CFparm['S'-'A']
#define WHENUNSP  CFparm['T'-'A']
#define UNLNKFILE CFparm['U'-'A']
#define PWIDTH    CFparm['W'-'A']
#define PRTITLE   CFparm['X'-'A']
#define ZOPTS     CFparm['Z'-'A']

/*
 * printjob return codes
 */
#define JBUSY   -1              /* being done by other means */
#define JFAIL    0              /* repeat with retry */
#define JSUCC    1              /* done */
#define JABORT   2              /* done, but problems */

/*
 * Mail program
 */
#ifndef MAIL
#define MAIL "/usr/lib/sendmail"
#endif /* MAIL */

/*
 * File Transfer Protocol Flags
 */
#define CEND 5		/* last file */
#define CNAME 4		/* control file */
#define DFILE 3		/* data file */
#define CFILE 2		/* control file */

/*
 * Local parameters to the spooling system
 * Defaults for line printer capabilities data base
 */
#define DEFLOGF		"log"
#define DEFLOCK		"lock"
#define DEFSTAT		"status"
#define DEFACCT		"acct"
#define	DEFFORMATS	"flp"	/* default formats */
#define	DEFMX		1000
#define	DEFRETRY	3		/* maximum number of retries */
#define DEFMAXCOPIES	0
#define DEFMINLEGAL	32	/* check only first 32 chars */
#define DEFFF		"\f"
#define DEFWIDTH	132
#define DEFLENGTH	66
#define DEFPRICE	20	/* cost in dollars per thousand pages */
#define DEFPRIORITY	'M'	/* default priority level */
#define DEFPR		"/bin/pr"	/* default pr program */

#ifndef DAEMON
#define DAEMON		"daemon"	/* daemon user id */
#endif

/*
 * REAL, not TEST version
 */

/*
 * path name of files created by lpd.
 */
#ifndef RMASTERLOCK
#define RMASTERLOCK	"/var/spool/lpd/lpd.lock."
#endif
#ifndef RDEFLPDLOGF
#define	RDEFLPDLOGF	"/var/spool/lpd/lpd.log."
#endif
#ifndef RPRINTCAP
#define RPRINTCAP	"/etc/printcap"
#endif
#ifndef RPERMFILE
#define	RPERMFILE	"/etc/printer_perms"
#endif

/*
 * connection services to be used
 */
#define SERVERNAME	"printer"
#define SERVERPROT	"tcp"

/*
 *	The remote client must call from a port in the following range.
 *	This is used for authentication purposes.  The ports less than
 *	IPPORT_RESERVED are accessible only to SU processes.
 */

#define RMINPORTNO	(IPPORT_RESERVED / 2)
#define RMAXPORTNO	(IPPORT_RESERVED - 1)

/*
 *	DEBUG Version, use to avoid screwing up normal operations
 * path name of files created by lpd.
 */
#ifndef TMASTERLOCK
#define TMASTERLOCK 	"/tmp/lpd.lock."
#endif
#ifndef TDEFLPDLOGF
#define	TDEFLPDLOGF	"/tmp/lpd.log."
#endif
#ifndef TPRINTCAP
#define TPRINTCAP	"/tmp/printcap."
#endif
#ifndef TPERMFILE
#define	TPERMFILE	"/tmp/printer_perms."
#endif
#ifndef TSERVICENAME
#define TSERVICENAME	"testlpr"
#endif
#ifndef TPORTNUM
#define TPORTNUM	1600	/* test port number */
#endif

/*
 *	The remote client must call from a port in the following range.
 *	This is used for authentication purposes.  The ports less than
 *	IPPORT_RESERVED are accessible only to SU processes.
 */
#define TMINPORTNO	(IPPORT_RESERVED+1)
#define TMAXPORTNO	(2*IPPORT_RESERVED)

#define SYSLOG_FACILITY	LOG_LPR		/* what facility to use, when supported */

/*************************************************************************
 * jmason additions from here on (v3.3.0)
 */

/* there's AF_UNIX support in there too, but what's the point of that!? */
#define SOCK_FAMILY	AF_INET
