/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 ***************************************************************************
 * MODULE: Print_stty.c
 * handle tty line settings to the printer -- isolated by <jmason@iona.ie>
 ***************************************************************************
 * int Do_stty(): set tty line settings
 ***************************************************************************/

# include "lp.h"

#ifdef USE_TERMIOS
# include <sys/termios.h>
#else
# include <sgtty.h>
#endif

struct bauds {
	char *string;
	int	baud;
	int	speed;
} bauds[] = {
	"110", 110,	B110,
	"134", 134,	B134,
	"150", 150,	B150,
	"300", 300,	B300,
	"600", 600,	B600,
	"1200", 1200,	B1200,
	"1800", 1800,	B1800,
	"2400", 2400,	B2400,
	"4800", 4800,	B4800,
	"9600", 9600,	B9600,
	"19200", 19200,	B19200,
	"38400", 38400,	B38400,
	(char *)0, 0,	0
};

#ifdef USE_TERMIOS
struct s_term_dat {
   char *name;
   uint or_dat;
   uint and_dat;
};

/* c_iflag bits */
struct s_term_dat c_i_dat[] = 
{
   {"IGNBRK",   IGNBRK, 0},
   {"-IGNBRK",  0, IGNBRK},
   {"BRKINT",   BRKINT, 0},
   {"-BRKINT",  0, BRKINT},
   {"IGNPAR",   IGNPAR, 0},
   {"-IGNPAR",  0, IGNPAR},
   {"PARMRK",   PARMRK, 0},
   {"-PARMRK",  0, PARMRK},
   {"INPCK",    INPCK, 0},
   {"-INPCK",   0, INPCK},
   {"ISTRIP",   ISTRIP, 0},
   {"-ISTRIP",  0, ISTRIP},
   {"INLCR",    INLCR, 0},
   {"-INLCR",   0, INLCR},
   {"IGNCR",    IGNCR, 0},
   {"-IGNCR",   0, IGNCR},
   {"ICRNL",    ICRNL, 0},
   {"-ICRNL",   0, ICRNL},
   {"IUCLC",    IUCLC, 0},
   {"-IUCLC",   0, IUCLC},
   {"IXON",     IXON, 0},
   {"-IXON",    0, IXON},
   {"IXANY",    IXANY, 0},
   {"-IXANY",   0, IXANY},
   {"IXOFF",    IXOFF, 0},
   {"-IXOFF",   0, IXOFF},

#ifdef IMAXBEL
   {"IMAXBEL",  IMAXBEL, 0},
   {"-IMAXBEL", 0, IMAXBEL},
#else
   /* work out what they do and simulate with IFLAGONE|IFLAGTWO */
#endif

   /* jmason addition: */
   {"PASS8", 0, ISTRIP},
   {"-PASS8", ISTRIP, 0},
   {NULL, 0, 0}
};

/* c_oflag bits */
struct s_term_dat c_o_dat[] = 
{
   {"OPOST",   OPOST, 0},
   {"-OPOST",  0, OPOST},
   {"OLCUC",   OLCUC, 0},
   {"-OLCUC",  0, OLCUC},
   {"ONLCR",   ONLCR, 0},
   {"-ONLCR",  0, ONLCR},
   {"OCRNL",   OCRNL, 0},
   {"-OCRNL",  0, OCRNL},
   {"ONOCR",   ONOCR, 0},
   {"-ONOCR",  0, ONOCR},
   {"ONLRET",  ONLRET, 0},
   {"-ONLRET", 0, ONLRET},
   {"OFILL",   OFILL, 0},
   {"-OFILL",  0, OFILL},
   {"OFDEL",   OFDEL, 0},
   {"-OFDEL",  0, OFDEL},
   {"NL0",     0, NLDLY},
   {"NL1",     NL1, NLDLY},
   {"CR0", CR0, CRDLY},
   {"CR1", CR1, CRDLY},
   {"CR2", CR2, CRDLY},
   {"CR3", CR3, CRDLY},
   {"TAB0", TAB0, TABDLY},
   {"TAB1", TAB1, TABDLY},
   {"TAB2", TAB2, TABDLY},
   {"TAB3", TAB3, TABDLY},
   {"BS0", BS0, BSDLY},
   {"BS1", BS1, BSDLY},
   {"VT0", VT0, VTDLY},
   {"VT1", VT1, VTDLY},
   {"FF0", FF0, FFDLY},
   {"FF1", FF1, FFDLY},

   /* jmason addition: */
   {"TABS", TAB3, TABDLY},
   {NULL, 0, 0}
};

/* c_cflag bit meaning */
struct s_term_dat c_c_dat[] = 
{
   {"B0",     B0, CBAUD},
   {"0",      B0, CBAUD},
   {"B50",    B50, CBAUD},
   {"50",     B50, CBAUD},
   {"B75",    B75, CBAUD},
   {"75",     B75, CBAUD},
   {"B110",   B110, CBAUD},
   {"110" ,   B110, CBAUD},
   {"B134",   B134, CBAUD},
   {"134" ,   B134, CBAUD},
   {"B150",   B150, CBAUD},
   {"150" ,   B150, CBAUD},
   {"B200",   B200, CBAUD},
   {"200" ,   B200, CBAUD},
   {"B300",   B300, CBAUD},
   {"300" ,   B300, CBAUD},
   {"B600",   B600, CBAUD},
   {"600",    B600, CBAUD},
   {"B1200",  B1200, CBAUD},
   {"1200 ",  B1200, CBAUD},
   {"B1800",  B1800, CBAUD},
   {"1800",   B1800, CBAUD},
   {"B2400",  B2400, CBAUD},
   {"2400",   B2400, CBAUD},
   {"B4800",  B4800, CBAUD},
   {"4800",   B4800, CBAUD},
   {"B9600",  B9600, CBAUD},
   {"9600",   B9600, CBAUD},
   {"B19200", B19200, CBAUD},
   {"19200",  B19200, CBAUD},
   {"B38400", B38400, CBAUD},
   {"38400",  B38400, CBAUD},
   {"EXTA",   EXTA, CBAUD},
   {"EXTB",   EXTB, CBAUD},
   {"CS5", CS5, CSIZE},
   {"CS6", CS6, CSIZE},
   {"CS7", CS7, CSIZE},
   {"CS8", CS8, CSIZE},
   {"CSTOPB", CSTOPB, 0},
   {"-CSTOPB", 0, CSTOPB},
   {"CREAD", CREAD, 0},
   {"-CREAD", 0, CREAD},
   {"PARENB", PARENB, 0},
   {"-PARENB", 0, PARENB},
   {"PARODD", PARODD, 0},
   {"-PARODD", 0, PARODD},
   {"HUPCL", HUPCL, 0},
   {"-HUPCL", 0, HUPCL},
   {"CLOCAL", CLOCAL, 0},
   {"-CLOCAL", 0, CLOCAL},

#ifdef CRTSCTS
   /* work out what they do and simulate with IFLAGONE|IFLAGTWO */
   {"CRTSCTS", CRTSCTS, 0},
   {"-CRTSCTS", 0, CRTSCTS},
#endif

   /* jmason additions, from SunOS stty(1v) combination modes: */
   {"EVENP", PARENB | CS7, PARODD | CSIZE},
   {"-EVENP", CS8, PARENB | CSIZE},
   {"PARITY", PARENB | CS7, PARODD | CSIZE},
   {"-PARITY", CS8, PARENB | CSIZE},
   {"ODDP", PARENB | CS7, CSIZE},
   {"-ODDP", CS8, PARENB | PARODD | CSIZE},
   {"PASS8", CS8, PARENB | CSIZE},
   {"-PASS8", PARENB | CS7, CSIZE},
   {NULL, 0, 0}
};

/* c_lflag bits */
struct s_term_dat c_l_dat[] = 
{
   {"ISIG", ISIG, 0},
   {"-ISIG", 0, ISIG},
   {"ICANON", ICANON, 0},
   {"-ICANON", 0, ICANON},
   {"XCASE", XCASE, 0},
   {"-XCASE", 0, XCASE},
   {"ECHO", ECHO, 0},
   {"-ECHO", 0, ECHO},
   {"ECHOE", ECHOE, 0},
   {"-ECHOE", 0, ECHOE},
   {"ECHOK", ECHOK, 0},
   {"-ECHOK", 0, ECHOK},
   {"ECHONL", ECHONL, 0},
   {"-ECHONL", 0, ECHONL},
   {"NOFLSH", NOFLSH, 0},
   {"-NOFLSH", 0, NOFLSH},
   {"TOSTOP", TOSTOP, 0},
   {"-TOSTOP", 0, TOSTOP},
   {"IEXTEN", IEXTEN, 0},
   {"-IEXTEN", 0, IEXTEN},

#ifdef ECHOCTL
   /* work out what they do and simulate with IFLAGONE|IFLAGTWO */
   {"ECHOCTL", ECHOCTL, 0},
   {"-ECHOCTL", 0, ECHOCTL},
   {"ECHOPRT", ECHOPRT, 0},
   {"-ECHOPRT", 0, ECHOPRT},
   {"ECHOKE", ECHOKE, 0},
   {"-ECHOKE", 0, ECHOKE},
   {"FLUSHO", FLUSHO, 0},
   {"-FLUSHO", 0, FLUSHO},
   {"PENDIN", PENDIN, 0},
   {"-PENDIN", 0, PENDIN},
#endif
   {NULL, 0, 0}
};

struct termios t_dat;

struct	special {
	char	*name;
	char	*cp;
} special[] = {
	"stop",		&(t_dat.c_cc[VSTOP]),
	"start",	&(t_dat.c_cc[VSTART]),
	0
};

Do_stty( fd )
	int fd;
{
	int i;
	char buf[BUFSIZ], *bp, *ep, *arg;

	if (
#ifdef USE_TCGETATTR
		tcgetattr(fd, &t_dat) < 0
#else
		ioctl(fd, TCGETS, &t_dat) < 0
#endif
			) {

		logerr_die( XLOG_INFO,"cannot get tty parameters");
	}
	if(Debug>3)log(XLOG_DEBUG,"stty: before iflag 0x%x, oflag 0x%x, cflag 0x%x lflag 0x%x",
			t_dat.c_iflag, t_dat.c_oflag, t_dat.c_cflag, t_dat.c_lflag);
	if( BR ) {
	  for(i=0; bauds[i].baud && BR != bauds[i].baud; i++);
	  if( i == 0){
		  fatal(XLOG_INFO,"illegal baud rate %d", BR);
	  }
          t_dat.c_cflag = (t_dat.c_cflag & ~CBAUD) | bauds[i].speed;
	}
	t_dat.c_cflag &= ~FC;
	t_dat.c_cflag |= FS;
	t_dat.c_lflag &= ~XC;
	t_dat.c_lflag |= XS;
 

	if( TY && *TY ){
		(void)strcpy(buf, TY);
		ep = buf;
	} else {
		ep = 0;
	}
	while( ep && *ep ){
		for( ; *ep && isspace(*ep) ; ++ ep );
		for( arg = ep; *ep && !isspace(*ep) ; ++ ep )
                {
                   *ep = toupper(*ep);
                }
		if( *ep ){
			*ep = 0;
			++ep;
		}

		for(i=0; c_i_dat[i].name && strcmp(c_i_dat[i].name,arg); i++);
		if(c_i_dat[i].name) {
			if(Debug>4)log(XLOG_DEBUG,
				"stty: c_iflag %s, mc 0x%x ms 0x%x",
				c_i_dat[i].name, c_i_dat[i].or_dat, c_i_dat[i].and_dat);
                        t_dat.c_iflag &= ~(c_i_dat[i].and_dat);
                        t_dat.c_iflag |= c_i_dat[i].or_dat;
			continue;
		}

		for(i=0; c_o_dat[i].name && strcmp(c_o_dat[i].name,arg); i++);
		if(c_o_dat[i].name) {
			if(Debug>4)log(XLOG_DEBUG,
				"stty: c_oflag %s, mc 0x%x ms 0x%x",
				c_o_dat[i].name, c_o_dat[i].or_dat, c_o_dat[i].and_dat);
                        t_dat.c_oflag &= ~(c_o_dat[i].and_dat);
                        t_dat.c_oflag |= c_o_dat[i].or_dat;
			continue;
		}

		for(i=0; c_c_dat[i].name && strcmp(c_c_dat[i].name,arg); i++);
		if(c_c_dat[i].name) {
			if(Debug>4)log(XLOG_DEBUG,
				"stty: c_cflag %s, mc 0x%x ms 0x%x",
				c_c_dat[i].name, c_c_dat[i].or_dat, c_c_dat[i].and_dat);
                        t_dat.c_cflag &= ~(c_c_dat[i].and_dat);
                        t_dat.c_cflag |= c_c_dat[i].or_dat;
			continue;
		}

		for(i=0; c_l_dat[i].name && strcmp(c_l_dat[i].name,arg); i++);
		if(c_l_dat[i].name) {
			if(Debug>4)log(XLOG_DEBUG,
				"stty: c_lflag %s, mc 0x%x ms 0x%x",
				c_l_dat[i].name, c_l_dat[i].or_dat, c_l_dat[i].and_dat);
                        t_dat.c_lflag &= ~(c_l_dat[i].and_dat);
                        t_dat.c_lflag |= c_l_dat[i].or_dat;
			continue;
		}

		for (i = 0; special[i].name && strcmp(special[i].name,arg); i++);
		if( special[i].name ){
			for( ; *ep && isspace(*ep) ; ++ ep );
			for( bp = ep; *ep && !isspace(*ep) ; ++ ep );
			if( *ep ){
				*ep = 0;
				++ep;
			}
			if( *bp == 0 ){
				fatal( XLOG_INFO, "stty: missing parameter for %s", arg );
			}
			if (bp[0] == '^'){
				if( bp[1] == '?' ){
					*special[i].cp = 0177;
				} else {
					*special[i].cp = 037 & bp[1];
				}
			} else {
				*special[i].cp = bp[0];
			}
			if(Debug>4)log(XLOG_DEBUG,"stty: special %s %s", arg, bp );
			continue;
		}
		fatal(XLOG_INFO,"unknown mode: %s\n", arg);
	}
	if(Debug>3)log(XLOG_DEBUG,"stty: after iflag 0x%x, oflag 0x%x, cflag 0x%x lflag 0x%x",
			t_dat.c_iflag, t_dat.c_oflag, t_dat.c_cflag, t_dat.c_lflag);

	if(
#ifdef USE_TCSETATTR
		tcsetattr(fd, TCSANOW, &t_dat) < 0
#else
		ioctl(fd, TCSETS, &t_dat) < 0
#endif
			) {
		logerr_die( XLOG_NOTICE,"cannot set tty parameters");
	}
}

#elif defined(USE_SGTTYB)

/***********************************************************************
 * BSD-based systems support
 ***********************************************************************/

/*
 * Fri Feb 26 08:44:53 CST 1988 Patrick Powell
 * set terminal modes
 * This code was based on a public domain version of public domain version
 * of stty. I suppose that I could have created if from scratermctrlh,
 * but I have seen the same table appearing
 * in many "public domain" display terminal modes programs.
 *
 * jmason: I've added support for some more recent combination modes,
 *     such as "pass8" and "parity".
 */

struct tchars termctrl;
struct ltchars linectrl;
struct sgttyb mode;
struct
{
	char	*string;
	int	set;
	int	reset;
	int	lset;
	int	lreset;
} modes[] = {
	"bs0",		BS0, BS1, 0, 0,
	"bs1",		BS1, BS1, 0, 0,
	"cbreak",	CBREAK, 0, 0, 0,
	"-cbreak",	0, CBREAK, 0, 0,
	"cooked",	0, RAW, 0, 0,
	"cr0",		CR0, CR3, 0, 0,
	"cr1",		CR1, CR3, 0, 0,
	"cr2",		CR2, CR3, 0, 0,
	"cr3",		CR3, CR3, 0, 0,
	"decctlq",	0, 0, LDECCTQ, 0,
	"-decctlq",	0, 0, 0, LDECCTQ,
	"echo",		ECHO, 0, 0, 0,
	"-echo",	0, ECHO, 0, 0,
	"even",		EVENP, 0, 0, 0,
	"-even",	0, EVENP, 0, 0,
	"ff0",		FF0, FF1, 0, 0,
	"ff1",		FF1, FF1, 0, 0,
	"lcase",	LCASE, 0, 0, 0,
	"-lcase",	0, LCASE, 0, 0,
	"litout",	0, 0, LLITOUT, 0,
	"-litout",	0, 0, 0, LLITOUT,
	"nl",		0, CRMOD, 0, 0,
	"-nl",		CRMOD, 0, 0, 0,
	"nl0",		NL0, NL3, 0, 0,
	"nl1",		NL1, NL3, 0, 0,
	"nl2",		NL2, NL3, 0, 0,
	"nl3",		NL3, NL3, 0, 0,
	"noflsh",	0, 0, LNOFLSH, 0,
	"-noflsh",	0, 0, 0, LNOFLSH,
	"nohang",	0, 0, LNOHANG, 0,
	"-nohang",	0, 0, 0, LNOHANG,
	"odd",		ODDP, 0, 0, 0,
	"-odd",		0, ODDP, 0, 0,
	"raw",		RAW, 0, 0, 0,
	"-raw",		0, RAW, 0, 0,
	"tab0",		TAB0, XTABS, 0, 0,
	"tab1",		TAB1, XTABS, 0, 0,
	"tab2",		TAB2, XTABS, 0, 0,
	"tabs",		0, XTABS, 0, 0,
	"-tabs",	XTABS, 0, 0, 0,
	"tandem",	TANDEM, 0, 0, 0,
	"-tandem",	0, TANDEM, 0, 0,
	"tilde",	0, 0, LTILDE, 0,
	"-tilde",	0, 0, 0, LTILDE,
	"tn300",	CR1, ALLDELAY, 0, 0,
	"tty33",	CR1, ALLDELAY, 0, 0,
	"tty37",	FF1+CR2+TAB1+NL1, ALLDELAY, 0, 0,
	"vt05",		NL2, ALLDELAY, 0, 0,

	/* jmason modes and synonyms: */
	"evenp",	EVENP, 0, 0, 0,
	"-evenp",	0, EVENP, 0, 0,
	"parity",	EVENP, 0, 0, 0,
	"-parity",	0, EVENP, 0, 0,
	"oddp",		ODDP, 0, 0, 0,
	"-oddp",	0, ODDP, 0, 0,
	"pass8",	0, 0, LPASS8, 0,
	"-pass8",	0, 0, 0, LPASS8,
};


struct	special {
	char	*name;
	char	*cp;
	char	def;
} special[] = {
	"stop",		&termctrl.t_stopc,		CSTOP,
	"start",	&termctrl.t_startc,		CSTART,
	0
};

Do_stty( fd )
	int fd;
{
	int i;
	int	localmode;
	int	linedisc;
	char buf[BUFSIZ], *bp, *ep, *arg;

	if( ioctl(fd, TIOCGETP, &mode) < 0
	 || ioctl(fd, TIOCGETC, &termctrl) < 0
	 || ioctl(fd, TIOCLGET, &localmode) < 0
	 || ioctl(fd, TIOCGLTC, &linectrl) < 0 ){
		logerr_die( XLOG_INFO,"cannot get tty parameters");
	}
	if(Debug>3)log(XLOG_DEBUG,"stty: before mode 0x%x, lmode 0x%x, speed 0x%x",
			mode.sg_flags, localmode, mode.sg_ispeed );
	if( BR ){
	  for(i=0; bauds[i].baud && BR != bauds[i].baud; i++);
	  if( i == 0){
		  fatal(XLOG_INFO,"illegal baud rate %d", BR);
	  }
	  mode.sg_ispeed = mode.sg_ospeed = bauds[i].speed;
	}
	mode.sg_flags &= ~FC;
	mode.sg_flags |= FS;
	localmode &= ~XC;
	localmode |= XS;
 

	if( TY && *TY ){
		(void)strcpy(buf, TY);
		ep = buf;
	} else {
		ep = 0;
	}
	while( ep && *ep ){
		for( ; *ep && isspace(*ep) ; ++ ep );
		for( arg = ep; *ep && !isspace(*ep) ; ++ ep );
		if( *ep ){
			*ep = 0;
			++ep;
		}
		for(i=0; modes[i].string && strcmp(modes[i].string,arg); i++);
		if(modes[i].string) {
			if(Debug>4)log(XLOG_DEBUG,
				"stty: modes %s, mc 0x%x ms 0x%x lc 0x%x ls 0x%x",
				modes[i].string, modes[i].reset, modes[i].set,
				modes[i].lreset, modes[i].lset );
			mode.sg_flags &= ~modes[i].reset;
			mode.sg_flags |= modes[i].set;
			localmode &= ~modes[i].lreset;
			localmode |= modes[i].lset;
			continue;
		}
		for (i = 0; special[i].name && strcmp(special[i].name,arg); i++);
		if( special[i].name ){
			for( ; *ep && isspace(*ep) ; ++ ep );
			for( bp = ep; *ep && !isspace(*ep) ; ++ ep );
			if( *ep ){
				*ep = 0;
				++ep;
			}
			if( *bp == 0 ){
				fatal( XLOG_INFO, "stty: missing parameter for %s", arg );
			}
			if (bp[0] == '^'){
				if( bp[1] == '?' ){
					*special[i].cp = 0177;
				} else {
					*special[i].cp = 037 & bp[1];
				}
			} else {
				*special[i].cp = bp[0];
			}
			if(Debug>4)log(XLOG_DEBUG,"stty: special %s %s", arg, bp );
			continue;
		}
		for(i=0; bauds[i].string && strcmp(bauds[i].string,arg); i++);
		if(bauds[i].string) {
			if(Debug>4)log(XLOG_DEBUG,"stty: speed %s", arg );
			mode.sg_ispeed = mode.sg_ospeed = bauds[i].speed;
			continue;
		}
		if (!strcmp("new", arg)){
			if(Debug>4)log(XLOG_DEBUG,"stty: ldisc %s", arg );
			linedisc = NTTYDISC;
			if (ioctl(fd, TIOCSETD, &linedisc)<0)
				logerr_die(XLOG_INFO,"stty: TIOCSETD ioctl failed");
			continue;
		}
		if (!strcmp("old",arg)){
			if(Debug>4)log(XLOG_DEBUG,"stty: ldisc %s", arg );
			linedisc = 0;
			if (ioctl(fd, TIOCSETD, &linedisc)<0)
				logerr_die(XLOG_INFO,"stty: TIOCSETD ioctl failed");
			continue;
		}
		fatal(XLOG_INFO,"unknown mode: %s\n", arg);
	}
	if(Debug>3)log(XLOG_DEBUG,"stty: after mode 0x%x, lmode 0x%x, speed 0x%x",
			mode.sg_flags, localmode, mode.sg_ispeed );
	if( ioctl(fd, TIOCSETN, &mode) < 0
		|| ioctl(fd, TIOCSETC, &termctrl) < 0
		|| ioctl(fd, TIOCSLTC, &linectrl) < 0
		|| ioctl(fd, TIOCLSET, &localmode) < 0 ){
		logerr_die( XLOG_NOTICE,"cannot set tty parameters");
	}
}
#else /* USE_SOMETHING_ELSE? */

Do_stty( fd )
	int fd;
{
	logerr_die( XLOG_NOTICE,"No method of sttying port!");
}

#endif
