/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 ***************************************************************************
 * MODULE: remoteprinter.c
 * send a job to a remote Printer
 ***************************************************************************/

#include "lp.h"

/***************************************************************************
 * int Setup_xfer()
 * sends the "\02Printer" message to the remote Host
 * Returns: successful JSUCC, otherwise JFAIL
 ***************************************************************************/
int
Setup_xfer()
{
	char buf[BUFSIZ];

	if( RP == 0 || *RP == 0 ){
		fatal(XLOG_INFO, "no RP specified for RM (%s)", RM );
	}
	(void)sprintf( buf, "%c%s\n",  REQ_RECV, RP );
	if(Debug>3)log(XLOG_DEBUG,
		"Setup_xfer: request %d, (REQ_RECV) Host '%s' pr '%s'",
		REQ_RECV,RM,RP);
	if( JSUCC == Link_line( -1, buf )){
		if(Debug>3)log(XLOG_DEBUG,"Setup_xfer: request sent");
		if( JSUCC == Link_confirm()){
			if(Debug>3)log(XLOG_DEBUG,"Setup_xfer: request to %s confirmed",RM);
			return(JSUCC);
		}
		if(Debug>3)log(XLOG_DEBUG,"Setup_xfer: request to %s not confirmed",RM);
	} else {
		if(Debug>3)log(XLOG_DEBUG,"Setup_xfer: request not sent");
	}
	setstatus("request for file transfer to %s failed %s", RM, Time_str() );
	fatal( XLOG_INFO, "Setup_xfer: cannot start transfer to %s", RM );
	/*NOTREACHED*/
}

/***************************************************************************
 * Sendjob( FILE *cfp; struct plp_queue *q)
 *   1.  First scan extracts information which controls printing
 *			and finds the names of the files to send.
 *	 2.	 The data files are then sent.
 *	 3.	 Control file is then sent.
 *   4.  Job files are removed
 *   5.  Control file is removed
 * Returns:  JBUSY, JFAIL, JSUCC, JABORT
 * NOTE: this code was based on a description of the LPD file transfer
 *    protocol.
 *
 * File transfer protocol:
 * Sender: sends line of form 'flag' length filename
 * Receiver: acknowledges with 0 (single byte)
 * Sender: sends file (length bytes of information), then a teminating 0
 * Receiver: acknowledges with 0 (single byte)
 *
 * Tne end is signalled by closing the stream, which results in a read error.
 * 
 *************************************************************************/
Sendjob(cfp, q)
	FILE *cfp;			/* control file */
	struct plp_queue *q;	/* job entry */
{
	FILE *fp;					/* File to send */
	char parm[MAXPATHLEN];			/* holds a line read in */
	char *arg;					/* control line argument */
	char opt;					/* control line option */
	int  jstatus;			    /* job status */
	int  i;						/* ACME Integers, Inc. */

	if(Debug>3)log(XLOG_DEBUG,"*** STARTING SENDJOB ***");
	/*
	 * set job status
	 */
	jstatus = JFAIL;	/* default */

	/*
	 * read the control file and extract user information
	 */
	Parmcount = 0;
	while (fgets (parm, sizeof(parm), cfp )) {
		if( (arg = index(parm, '\n')) == 0 ){
			log( XLOG_INFO, "Sendjob: bad control file format (%s), no endline",
				q->q_name);
			jstatus = JABORT;
			goto error;
		}
		*arg = 0;
		opt = parm[0];
		arg = parm+1;
		if( !isascii(opt) || !isalnum( opt ) || strlen( arg ) > MAXPARMLEN ){
			log( XLOG_INFO, "Sendjob: bad control file (%s), line(%s)",
				q->q_name,parm);
			jstatus = JABORT;
			goto error;
		}
		if( !islower( opt ) && opt != 'U' ){
			continue;
		}
		if( Job_match( q->q_name, arg ) == 0 ){
			log( XLOG_INFO, "Sendjob: bad file name format (%s)", arg );
			jstatus = JABORT;
			goto error;
		}
		if( islower( opt )){
			if( Add_name(arg) < 0 ){
				log( XLOG_INFO, "Sendjob: too many files (%s)", arg );
				jstatus = JABORT;
				goto error;
			}
		} else {
			if( (i = Find_name(arg) ) < 0 ){
				log(XLOG_INFO,"Sendjob: job(%s)U specified for '%s' not in job",
					q->q_name,parm);
				jstatus = JABORT;
				goto error;
			}
		}
	}
	/*
	 * We send the control file first if we are using the PLP protocol
	 */
	if( FJ) {
		jstatus = sendfile( cfp, q->q_name, CNAME);
		if( jstatus != JSUCC ){
			setstatus("transfer %s to %s failed %s",q->q_name,RM,Time_str());
			log(XLOG_INFO,"Sendjob: sendname %s to %s failed",q->q_name,RM);
			goto error;
		}
	}
	/*
	 * send the jobs in the file
	 */
	for( i = 0; i < Parmcount; ++i) {
		arg = Parms[i].filename;
		/*
		 * send the file
		 */
		if(Debug>3)log(XLOG_DEBUG,"Sendjob: sending file %s", arg );
		jstatus = senddatafile( arg, DFILE );

		if( jstatus != JSUCC) {
			goto error;
		}
	}
	/*
	 * send the control file
	 */
	if(Debug>3)log(XLOG_DEBUG,"sending file %s", q->q_name );
	if( FJ ){
		jstatus = sendname( cfp, q->q_name, CEND );
	} else {
		jstatus = sendfile( cfp, q->q_name, CFILE );
	}
	if( jstatus != JSUCC) {
		setstatus("transfer %s to %s failed %s",q->q_name,RM,Time_str());
		log(XLOG_INFO,"Sendjob: transfer %s to host %s failed",q->q_name,RM);
		goto error;
	}
	/*
	 * finished!
	 */
	jstatus = JSUCC;
error:
	if( jstatus != JSUCC ){
		Link_close();
	}
	if(Debug>3)log(XLOG_DEBUG,"*** ENDING SENDJOB ***");
	return (jstatus);
}

/***************************************************************************
 * sendfile( FILE* fp; char *name; int flag )
 *   sendfile protocol:
 * 1. line containing the flag, number of bytes, and file name is sent.
 *     This has the format:
 *        "%c%d %s", flag, filesize, name
 *    DFILE (data file) flag is 03,  a CFILE (control file) flag is 02
 * 2. the remote end responds by sending back a single 0 character;
 *    anything else is an error condition and terminates transfer
 * 3. the file is then copied to the remote end and terminated with 0
 * 4. if an error is detected, we send a non-zero character
 *    and close the link.
 ***************************************************************************/

sendfile( fp, name, flag )
	FILE *fp;
	char *name;
	int flag;
{
	int succ;			/* ACME Integer, Inc. */
	struct stat statb;	/* status buffer */

	/*
	 * stat the file
	 */
	if( fstat( fileno(fp), &statb) < 0 ){
		logerr( XLOG_INFO, "sendname: could not stat %s", name );
		return( JFAIL );
	}
	/*
	 * rewind the file
	 */
	if( fseek(fp, 0L, 0) < 0 ){
		logerr_die( XLOG_INFO, "sendfile: fseek failed '%s'", name );
	}
	succ = sendname( fp, name, flag );
	if( succ == JSUCC ){
		succ = Link_copy( fp, (long)statb.st_size, name );
	}
	if( succ == JSUCC ){
		succ = Link_ack( 0 );
	}
	if( succ == JSUCC ){
		succ = Link_confirm();
	}
	if( succ != JSUCC ){
		succ = JFAIL;
		Link_close();
	}
	return( succ );
}

senddatafile(name, flag )
	char *name;
	int flag;
{
	int succ;			/* ACME Integer, Inc. */
	FILE *raw, *filtered;
	char *filter, tmpfile[BUFSIZ];

	if( (raw = fopen (name, "r")) == NULL ){
		logerr( XLOG_INFO,"senddatafile: cannot open data file %s",name);
		setstatus("cannot open data file %s",name);
		return(JABORT);
	}

	if( IF == 0 || *IF == 0) {	/* non-filtered -- very boring */
		succ = sendfile(raw, name, flag);
		fclose(raw);
		if (succ != JSUCC) {
			setstatus("transfer %s to %s failed %s",name,RM,Time_str());
			log(XLOG_INFO,"Sendjob: transfer %s to host %s failed",name,RM);
		}
		return(succ);
	}

	/* this remote printer has a text filter... thrills 'n' spills! */

	if (Debug>1) log(XLOG_DEBUG, "senddatafile: IF: starting filter");

	filter = Setup_filter('f', IF);
	if (filter == 0 || *filter==0) {
	    log( XLOG_INFO,"senddatafile: no filter, file \"%s\"", name);
	    return(JABORT);
	}

	(void) strcpy(tmpfile, name);
	tmpfile[0] = 't';			/* indicate tmp file */

	/* set up an fd, supposedly pointing to the printer, but
	 * really pointing to a temporary output file.
	 */
	if ((Print_fd = Open_tmpfile(tmpfile)) < 0) {
	    return(JFAIL);		/* already logged, just return */
	}

	if(Debug>0)log( XLOG_DEBUG,"senddatafile: starting filter %s", filter);

	succ = Print_filter(fileno(raw), filter, 0);	/* non-local printer mode */
	close(Print_fd);	/* close the output file */
	fclose(raw);		/* close the input file */

	/* if the filter failed, give up here. */
	if (succ != JSUCC) {
	    unlink(tmpfile);
	    setstatus("filter exited with bad status");
	    log (XLOG_INFO, "%s: filter exited with bad status", Printer);
	    return(succ);
	}

	/* otherwise, send the filtered output to the remote end */

	if (Debug>1) log(XLOG_DEBUG, "senddatafile: sending file %s, name %s, flag %d",
		tmpfile, name, flag);
	raw = fopen(tmpfile, "r");
	succ = sendfile(raw, name, flag);

	unlink(tmpfile);
	return(succ);
}

int Open_tmpfile(buf, realname)
    char *buf, *realname;
{
    int fd;

#if 0
    if ((fd = add_recfiles(buf)) < 0) {
	log( XLOG_INFO, "Open_tmpfile: too many files in recfiles");
	return(-1);
    }
#endif

    if((fd = Exlockcf(buf)) < 0) {
	logerr_die( XLOG_INFO, "Open_tmpfile: could not open and lock %s", buf );
    }

    if(Debug>3)log(XLOG_DEBUG,"Open_tmpfile: file %s, fd %d", buf, fd);
    return( fd );
}

/***************************************************************************
 * Send_error()
 * Called when you have retried a couple of times; bad job, needs attention
 ***************************************************************************/
void
Send_error()
{
	log(XLOG_CRIT, "job cannot be sent from %s to %s", Host, RM );
	exit( 0 );
}

/***************************************************************************
 * sendname( FILE *fp; char *name; int flag )
 *   sendname protocol:
 * 1. line containing the flag and file name is sent
 *     This has the format:
 *        "%c%d %s", flag, filesize, name
 *    DFILE (data file) flag is 03,  a CFILE (control file) flag is 02
 * 2. the remote end responds by sending back a single 0 character;
 *    anything else is an error condition and terminates transfer
 ***************************************************************************/
sendname( fp, name, flag )
	FILE *fp;
	char *name;
	int flag;
{
	struct stat statb;	/* status buffer */
	int succ;			/* ACME Integer, Inc. */
	char buf[BUFSIZ];

	/*
	 * stat the file
	 */
	if( fstat( fileno(fp), &statb) < 0 ){
		logerr( XLOG_INFO, "sendfile: could not stat %s", name );
		return( JFAIL );
	}
	(void)sprintf( buf, "%c%d %s\n", flag, statb.st_size, name);
	succ = Link_send( buf );
	if( succ == JSUCC ){
		succ = Link_confirm();
	}
	return( succ );
}

/***************************************************************************
 * Allsent( )
 *    all files have been sent; send a closing 0 and shut down link
 ***************************************************************************/
void
Allsent()
{
	(void)Link_ack( 0 );
	Link_close();
}
