/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 * version 3.3.0 Justin Mason July 1994
 ***************************************************************************
 * MODULE: sendmail.c
 * send mail to user on completion of a job
 ***************************************************************************/

#include "lp.h"

/*
 * sendmail ---
 *   tell people about job completion
 * 1. fork a sendmail process
 * 2. if successful, send the good news
 * 3. if unsuccessful, send the bad news
 */
sendmail(q, status)
	struct plp_queue *q;
	int status;
{
	static int p[2];	/* pipe */
	int i;				/* ACME Integer, Inc. */
	char buf[100];
	FILE *mail;			/* mail file */
	int pid;			/* sendmail process */
#if defined(INT_STATUS)
	int stat;
#else
	union wait stat;	/* daughter status */
#endif

	(void)sprintf(buf, "%s %s", MAIL, MAILNAME );
	if(Debug>3)log( XLOG_DEBUG, "mail command: %s", buf );
	if( pipe(p) < 0 ){
		logerr_die( XLOG_NOTICE, "pipe failed for sendmail" );
	}
	/*
	 * start up sendmail process
	 */
	if((pid = fork()) == 0){
		if( p[0] ){
			if( dup2(p[0], 0) < 0 ){
				logerr_die( XLOG_NOTICE, "dup2 failed in sendmail child" );
			}
			(void)close(p[0]);
		}
		(void)close(p[1]);

		/* run as "daemon" user so headers are legal */
                daemon_to_root(); full_root_to_daemon();
		secure_exec(buf);
		logerr_die(XLOG_INFO, "exec mail failed: %s", buf);
	} else if(pid < 0) {				/* parent */
		logerr_die( XLOG_NOTICE, "sendmail: fork failed" );
	}
	(void)close(p[0]);
	if( (mail = fdopen(p[1], "w")) == NULL ){
		logerr_die( XLOG_NOTICE, "sendmail: fdopen failed" );
	}
	(void)fprintf(mail,"To: %s@%s\n", q->q_user, &q->q_from);
	(void)fprintf(mail,"Subject: %s printer job\n\n", Printer);
	(void)fprintf(mail,"Your %s printer job %d (%s)",Printer,
		q->q_num, q->q_data);
	switch(  status ){
		case JSUCC:
			(void)fprintf( mail, " was successful.\n" );
			break;
		case JFAIL:
			(void)fprintf( mail, " failed, and retry count was exceeded.\n" );
			(void)fprintf( mail, "%s\n", Last_errormsg );
			break;
		default:
			(void)fprintf( mail, " died a horrible death.\n");
			(void)fprintf( mail, "%s\n", Last_errormsg );
			break;
	}
	(void) fflush(mail);
	(void) fclose(mail);
	while ((i = wait(&stat)) > 0 && i != pid){
		if(Debug>3)log( XLOG_DEBUG, "sendmail: caught %d, (%s)", i,
			Decode_status(&stat));
	}
	if(Debug>3)log( XLOG_DEBUG, "sendmail: %d finished (%s)",
		i,Decode_status(&stat) );
}
