/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 ***************************************************************************
 * MODULE: termclear.c
 ***************************************************************************
 * Term_clear()
 * 1. Do the initialization if neccessary
 * 2. clear the screen using the termcap information
 * This code was taken from "Screen Updating and Cursor Movement Optimization
 * A Library Package", by Ken. C. Arnold;  with 4.3BSD Release,
 * Programmers Reference Manual
 ***************************************************************************/
static int tinit_done;
static int is_term;

#include "config.h"

#ifdef TERMCAP
#include <stdio.h>
#include <sys/types.h>

#ifdef USE_SGTTYB
#include <sgtty.h>
#endif

#include <curses.h>

char *getenv();
extern int XLOG_INFO;
/***************************************************************************
 * definitions for TERMCAP
 ***************************************************************************/
char bp[1024];
char xp[1024];
char *area = xp;

#ifdef USE_OSPEED		/* we don't and it conflicts on sol2 */
#if defined(USE_TERMIOS)
speed_t ospeed;
struct termios sbuf;
#else
short ospeed;
struct sgttyb sbuf;
#endif
#endif	/* ospeed */

char *TI;
char *TE;
char *CL;
char *tgetstr();

pputs( c )
	int c;
{
	putchar( c );
}


Term_clear()
{
	char *sp;
	if( tinit_done == 0 ){
		tinit_done = 1;
		/*
		 * set things up
		 */
		if( isatty(0) && (sp = getenv( "TERM" )) != NULL ){
#ifdef USE_OSPEED		/* we don't and it conflicts on sol2 */
#if defined(USE_TERMIOS)
			/* get the speed */
			if( ioctl(1, TCGETS, &sbuf) < 0 ){
				logerr_die( XLOG_INFO,"ioctl on output failed" );
			}
			ospeed = sbuf.c_cflag & CBAUD;
#else
			/* get the speed */
			if( ioctl(1, TIOCGETP, &sbuf) < 0 ){
				logerr_die( XLOG_INFO,"ioctl on output failed" );
			}
			ospeed = sbuf.sg_ospeed;
#endif
#endif	/* ospeed */
			if( tgetent( bp, sp ) > 0 ){
				TI = tgetstr( "ti", &area );
				TE = tgetstr( "te", &area );
				CL = tgetstr( "cl", &area );
			}
			if( TI ){
				tputs( TI, 0, pputs );
			}
			is_term = 1;
		}
	}
	if( is_term && CL ){
		tputs( CL, 0, pputs );
	}
}

Term_finish()
{
	if( is_term && TE ){
			tputs( TE, 0, pputs );
	}
}
#else TERMCAP
Term_clear()
{ ; }
Term_finish()
{ ; }
#endif TERMCAP
