/***************************************************************************
 * U. Minnesota LPD Software * Copyright 1987, 1988, Patrick Powell
 ***************************************************************************
 * MODULE: lprm.c
 * lprm- remove files
 ***************************************************************************/

#include "lp.h"
#include "patchlevel.h"

extern SIG_RETURN_TYPE cleanup();

main(argc, argv)
	int argc;
	char **argv;
{
	struct passwd *pw_ent;		/* user entry in /etc/passwd */

	/*
	 * set umask to avoid problems with user umask
	 */
	(void)umask(0);
	/*
	 * Set fd 0, 1, 2 to /dev/null if not open
	 */
	Std_environ();
#	ifdef XPERIMENT
	Setup_test();
#	endif XPERIMENT

	lose_insecure_envs();   /* get ourselves a secure environment */
	set_user_uid(0); root_to_user();

	/*
	 * set up the pathnames for information files
	 */
	Tailor_names();
	/*
	 * set up the From information
	 */
	From = Host;
	/*
	 * get the user information
	 */
	if( (pw_ent = getpwuid( getuid() )) == 0 ){
		logerr_die( XLOG_INFO, "getpwuid failed on uid %d", getuid());
	}
	(void)strcpy( LOGNAME, pw_ent->pw_name );
	Person = LOGNAME;
	if( getuid() == 0 ){
		/* we are being invoked by root */
		Is_root = 1;
	}
	/*
	 * setup parameters
	 */
	Lprm_parms(argc, argv);
	/*
	 * set signals
	 */
	(void)signal(SIGPIPE, SIG_IGN);
	(void)signal(SIGHUP, cleanup);
	(void)signal(SIGINT, cleanup);
	(void)signal(SIGQUIT, cleanup);
	(void)signal(SIGTERM, cleanup);
	if( Parmcount != 0 ){
		rmjob();
	}
	exit( 0 );
}

/***************************************************************************
 * cleanup()
 * remove the temp files
 ***************************************************************************/

SIG_RETURN_TYPE cleanup()
{
	exit( 1 );
}

/***************************************************************************
 * Lprm_parms( int arc, char **argv )
 * 1. pick off the options
 * 2. set up the parameters
 ***************************************************************************/

static char *optstr = "P:D:Xa";
static int Xpert;
static int all_flag;
Lprm_parms( argc, argv )
	int argc;
	char **argv;
{
	int option;
	char *s;
	int i;

	while( (option = Getopt(argc,argv,optstr)) != EOF ){
		switch( option ){
		case 'D':
			if(Debug){
				Diemsg("Duplicate -D option");
			}
			if( sscanf( Optarg, "%d", &Debug ) != 1 || Debug <= 0){
				Diemsg("-D parameter is not positive integer" );
			}
			break;
		case 'X':
			if( Xpert ){
				Diemsg("Duplicate -X option");
			}
#			ifdef DEBUG
				Setup_test();
				Tailor_names();
#			else
				Diemsg( "-X not allowed" );
#			endif DEBUG
			break;
		case 'P':
			if( Printer ){
				Diemsg("Duplicate -P option");
			}
			Printer = Optarg;
			break;
		case 'a':
			if( all_flag ){
				Diemsg("Duplicate -a option");
			}
			all_flag = 1;
			break;
		case '?':
			break;
		default:
			fatal(XLOG_INFO, "Lprm_parms: badparm %c", option );
		}
	}

	/*
	 * set up the Parms[] array
	 */
	for( ; Optind < argc; ++Optind ){
		if( Parmcount < MAXPARMS ){
			s = argv[Optind];
			Parms[Parmcount].str = s;
			if( isdigit( *s )){
				Parms[Parmcount].num = atoi(s);
			} else {
				Parms[Parmcount].num = -1;
			}
			++Parmcount;
		} else {
			Diemsg( "too many files to print; break job up" );
		}
	}
	/*
	 * get the default printer
	 */
	Get_Printer(0);
	/*
	 * check for the all option
	 */
	if( all_flag ){
		if( Parmcount > 0 ){
			Diemsg("-a cannot be combined with other options" );
		}
		Parms[0].str = "-all";
		Parmcount = 1;
	}
	if(Debug>4){
		(void)fprintf(stdout,"Printer %s, Parmcount %d ", Printer, Parmcount );
		for( i = 0; i < Parmcount; ++i )
			(void)fprintf(stdout," '%s'(%d)",Parms[i].str, Parms[i].num ); 
		(void)fprintf(stdout,"\n"); (void)fflush(stdout);
	}
}
